use anyhow::{Context as _, Result, bail};

fn main() -> Result<()> {
    use std::io::IsTerminal as _;

    let is_terminal = std::io::stdout().is_terminal();
    let mut args = std::env::args();
    let _exe_name = args.next().context("How do we not have an exe name?")?;
    let mut filenames = vec![];
    for arg in args {
        filenames.push(arg);
    }

    let app = App {
        filenames,
        is_terminal,
    };
    app.run()
}

struct App {
    filenames: Vec<String>,
    is_terminal: bool,
}

impl App {
    fn run(&self) -> Result<()> {
        if self.filenames.is_empty() {
            return self.ls(".");
        }
        for filename in &self.filenames {
            let metadata = std::fs::metadata(&filename)?;
            if metadata.is_dir() {
                self.ls(filename)?;
            } else {
                self.cat(filename)?;
            }
        }
        Ok(())
    }

    fn cat(&self, filename: &str) -> Result<()> {
        // Use `/usr/bin` just in case for some weird reason we're symlinked as cat,
        // we don't want to recurse into ourselves
        let status = std::process::Command::new("/usr/bin/cat").arg(filename).status()?;
        if !status.success() {
            bail!("subprocess `cat` failed");
        }
        Ok(())
    }

    fn ls(&self, filename: &str) -> Result<()> {
        // Use `/usr/bin` just in case for some weird reason we're symlinked as ls,
        // we don't want to recurse into ourselves
        let mut command = std::process::Command::new("/usr/bin/ls");
        if self.is_terminal {
            command.arg("--color=always");
        }
        let status = command.arg(filename).status()?;
        if !status.success() {
            bail!("subprocess `ls` failed");
        }
        Ok(())
    }
}
